# DashbaordHatenaTrends plugin for Movable Type
# Author: Makoto Kawasaki (http://makotokw.com/weblog/)

package MT::Plugin::DashboardHatenaTrends;

use strict;
use warnings;

use MT;
use MT::Log;
use base qw(MT::Plugin);

use Date::Simple;
use XML::Simple;
use HTTP::Request;

my $plugin = MT::Plugin::DashboardHatenaTrends->new({
    id => 'proj.movabletype.dashboardhatenatrends',
    label => 'Hatena Trends',
    version => 0.1,
    description => 'Displays Hatene Hot Keywords',
    author => 'Makoto Kawasaki',
    name => 'Hatena Trends for MT4 Dashboard',
    author_link => 'http://makotokw.com/weblog/',
    blog_config_template => <<TMPL,
<dl>
<dt>display count(1-100)</dt>
<dd><input name="hhk_displaycount" size="20" value="<mt:var name="hhk_displaycount">"/></dd>
</dl>
TMPL
    settings => new MT::PluginSettings([
        ['hhk_displaycount'],
    ]),
});

MT->add_plugin($plugin);
sub instance { $plugin }

sub init_registry {
    my $plugin = shift;
    $plugin->registry({
        applications => {
            cms => {
                widgets => {
                    DashboardHatenaTrends => {
                        label => 'Hatena Trends',
                        template => 'hatenatrends.tmpl',
                        set => 'sidebar',
                        singular => 1,
                        handler => \&_hdlr_hatenatrends_widget,
                    },
                },
            }
        },
    });
}

our $hhk_feed_url = 'http://d.hatena.ne.jp/hotkeyword?mode=rss';
sub _hdlr_hatenatrends_widget {
    my $app = shift;
    my ($tmpl, $param) = @_;
    
    my $q = $app->param;
    my $blog_id = $q->param('blog_id');
    my $config = $plugin->get_config_hash('blog:'.$blog_id); 
    my $displaycount = $config->{hhk_displaycount};
    
    my $trycount = 7; # try 7 days
    my $trydate = Date::Simple->new;
    my $ua = $app->new_ua;
    
    $trydate = $trydate;
    while ($trycount>0)
    {
        my $feedurl = $hhk_feed_url.'&Date='.$trydate->format("%Y-%m-%d");
        my $req = HTTP::Request->new( GET => $feedurl );
        my $res = $ua->request($req);
        if ($res->is_success() && $res->code==200) 
        {
            my $xml = $res->content;
            my $xs = new XML::Simple();
            my $ref = $xs->XMLin($xml);
            my $items = $ref->{item};
            if ($displaycount>0 && $displaycount<=100) {
                @{$param->{keywords}} = @{$ref->{item}}[0..$displaycount-1];
            } else {
                $param->{keywords} = $ref->{item};
            }
             $param->{date} = $trydate->format("%Y.%m.%d");
            last;
        }
        $trydate = $trydate->prev;
        $trycount--;
    }
    1;
}

1;
